@@ -48,8 +48,11 @@ from setuptools.dist import Distribution
 # result for pip.
 # Also update tensorflow/tensorflow.bzl and
 # tensorflow/core/public/version.h
-_VERSION = '2.18.0'
+_VERSION = '2.19.0'
 
+# Update this version when a new libtpu stable version is released.
+LATEST_RELEASE_LIBTPU_VERSION = '0.0.10'
+NEXT_LIBTPU_VERSION = '0.0.11'
 
 # We use the same setup.py for all tensorflow_* packages and for the nightly
 # equivalents (tf_nightly_*). The package is controlled from the argument line
@@ -111,9 +114,9 @@ REQUIRED_PACKAGES = [
     # 'keras' on the release branch after the branch cut.
     'tensorboard',
     'keras',
-    # TODO(b/367877753): Update the upper bound to <2.2.0 once the compatibility
-    # issues with numpy 2.1.0 is fixed.
     'numpy',
+    'numpy >= 1.26.0; python_version<="3.12"',
+    'numpy~=2.1.0; python_version>="3.13"',
     'h5py',
     'ml_dtypes',
 ]
@@ -169,7 +172,6 @@ DOCLINES = __doc__.split('\n')
 
 # pylint: disable=line-too-long
 CONSOLE_SCRIPTS = [
-    'toco_from_protos = tensorflow.lite.toco.python.toco_from_protos:main',
     'tflite_convert = tensorflow.lite.python.tflite_convert:main',
     'toco = tensorflow.lite.python.tflite_convert:main',
     'saved_model_cli = tensorflow.python.tools.saved_model_cli:main',
@@ -250,7 +252,7 @@ class InstallHeaders(Command):
         '/tensorflow/include/external/com_google_absl': '',
         '/tensorflow/include/external/ducc': '/ducc',
         '/tensorflow/include/external/eigen_archive': '',
-        '/tensorflow/include/external/ml_dtypes': '/ml_dtypes',
+        '/tensorflow/include/external/ml_dtypes_py': '',
         '/tensorflow/include/tensorflow/compiler/xla': (
             '/tensorflow/include/xla'
         ),
@@ -313,17 +315,14 @@ if '_tpu' in project_name:
   # timing of these tests, the UTC date from eight hours ago is expected to be a
   # valid version.
   _libtpu_version = standard_or_nightly(
-      _VERSION.replace('-', ''),
-      '0.1.dev'
+      LATEST_RELEASE_LIBTPU_VERSION,
+      NEXT_LIBTPU_VERSION + '.dev'
       + (
           datetime.datetime.now(tz=datetime.timezone.utc)
           - datetime.timedelta(hours=8)
-      ).strftime('%Y%m%d'),
+      ).strftime('%Y%m%d') + '+nightly',
   )
-  if _libtpu_version.startswith('0.1'):
-    REQUIRED_PACKAGES.append([f'libtpu-nightly=={_libtpu_version}'])
-  else:
-    REQUIRED_PACKAGES.append([f'libtpu=={_libtpu_version}'])
+  REQUIRED_PACKAGES.append([f'libtpu=={_libtpu_version}'])
   CONSOLE_SCRIPTS.extend([
       'start_grpc_tpu_worker = tensorflow.python.tools.grpc_tpu_worker:run',
       ('start_grpc_tpu_service = '
@@ -361,7 +360,7 @@ headers = (
     + list(find_files('*.inc', 'tensorflow/include/external/com_google_absl'))
     + list(find_files('*.h', 'tensorflow/include/external/ducc/google'))
     + list(find_files('*', 'tensorflow/include/external/eigen_archive'))
-    + list(find_files('*.h', 'tensorflow/include/external/ml_dtypes'))
+    + list(find_files('*.h', 'tensorflow/include/external/ml_dtypes_py'))
 )
 
 # Quite a lot of setup() options are different if this is a collaborator package
